import json
from contextlib import suppress

import requests
from bs4 import BeautifulSoup

from zdict.dictionary import DictBase
from zdict.exceptions import NotFoundError
from zdict.models import Record


class iTaigiDict(DictBase):

    HOMEPAGE_URL = "https://itaigi.tw/"
    API = 'https://itaigi.tw/平臺項目列表/揣列表?關鍵字={word}'

    @property
    def provider(self):
        return 'itaigi'

    @property
    def title(self):
        return 'iTaigi - 愛台語'

    def _get_url(self, word) -> str:
        return self.API.format(word=word)

    def _get_word_text(self, word) -> str:
        try:
            text = word["文本資料"]
        except AttributeError:
            raise
        return text

    def _get_word_pronounce(self, word) -> str:
        try:
            pronounce = word["音標資料"]
        except AttributeError:
            raise
        return pronounce

    def _get_word_sentences(self, text, pronounce) -> dict:
        API = (
            "https://xn--fsqx9h.xn--v0qr21b.xn--kpry57d/%E7%9C%8B/"
            "?%E6%BC%A2%E5%AD%97={text}&%E8%87%BA%E7%BE%85={pronounce}"
        )
        url = API.format(text=text, pronounce=pronounce)

        try:
            _ = json.loads(requests.get(url).text)
        except Exception:
            return {}

        mandarin_sentence = None
        with suppress(KeyError, IndexError):
            mandarin_sentence = _["例句"][0]["華語"]

        chinese_sentence = None
        with suppress(KeyError, IndexError):
            chinese_sentence = _["例句"][0]["漢字"]

        taiwanese_sentence = None
        with suppress(KeyError, IndexError):
            taiwanese_sentence = _["例句"][0]["臺羅"]

        d = {
            'mandarin': mandarin_sentence,
            'chinese': chinese_sentence,
            'taiwanese': taiwanese_sentence,
        }
        return d

    def query(self, word: str):
        webpage = self._get_raw(word)
        soup = BeautifulSoup(webpage, "html.parser")
        response = json.loads(soup.text)

        # Not Found
        if not response.get("列表"):
            raise NotFoundError(word)

        # Show Chinese word from iTaigi in stead of user input if possible
        with suppress(KeyError, IndexError):
            word = response["列表"][0]["外語資料"]

        content = {}

        # Fetch basic words with text, pronounce and sentence
        try:
            basic_words = response["列表"][0]["新詞文本"]
        except Exception:
            raise
        else:
            content['basic_words'] = []
            for basic_word in basic_words:
                d = {}

                text = self._get_word_text(basic_word)
                d['text'] = text

                pronounce = self._get_word_pronounce(basic_word)
                d['pronounce'] = pronounce

                if self.args.verbose:
                    sentences = self._get_word_sentences(text, pronounce)
                    d['sentences'] = sentences

                content['basic_words'].append(d)

            # Fix issue-452 for iTaigi testings
            # iTaigi returns basic_words in random order.
            # Since we store basic_words in a list,
            # We have to sort it before saving into database
            # or the unit-testings would fail.
            content['basic_words'].sort(key=lambda word: word['text'])

        # Fetch related words
        try:
            related_words = response["其他建議"]
        except Exception:
            raise
        else:
            content['related_words'] = []
            for related_word in related_words:
                d = {}

                text = self._get_word_text(related_word)
                d['text'] = text

                pronounce = self._get_word_pronounce(related_word)
                d['pronounce'] = pronounce

                if self.args.verbose:
                    sentences = self._get_word_sentences(text, pronounce)
                    d['sentences'] = sentences

                content['related_words'].append(d)

        # Save content with word and provider.
        record = Record(
            word=word,
            content=json.dumps(content),
            source=self.provider,
        )

        return record

    def _show_word_sentences(self, word):
        if not word.get('sentences'):
            return
        if not any(word['sentences'].values()):
            return

        self.color.print('例句', color='lred', indent=4)

        if word['sentences']['mandarin']:
            self.color.print(
                word['sentences']['mandarin'], color='lyellow', indent=6,
            )
        if word['sentences']['chinese']:
            self.color.print(word['sentences']['chinese'], indent=6)

        if word['sentences']['taiwanese']:
            self.color.print(
                word['sentences']['taiwanese'], color='lwhite', indent=6,
            )

        self.color.print()

    def show(self, record: Record):
        '''
        color set:
            - mandarin: lyellow
            - chinese: normal
            - taiwanese: lwhite
        '''
        content = json.loads(record.content)

        # print the word we looked up
        self.color.print(record.word, 'lyellow')

        # print basic_words
        if content.get('basic_words'):
            for basic_word in content['basic_words']:
                self.color.print(basic_word['text'], end=' ', indent=2)
                self.color.print(basic_word['pronounce'], 'lwhite')

                if self.args.verbose:
                    self._show_word_sentences(basic_word)

        # print related_words if possible
        if content.get('related_words'):
            self.color.print()
            self.color.print("相關字詞", 'lred', indent=2)

            for related_word in content['related_words']:
                self.color.print(related_word['text'], end=' ', indent=4)
                self.color.print(related_word['pronounce'], 'lwhite')

                if self.args.verbose:
                    self._show_word_sentences(related_word)



import json

from bs4 import BeautifulSoup

from zdict.dictionary import DictBase
from zdict.exceptions import NotFoundError
from zdict.models import Record


# Change `Template` to the name of new dictionary. like xxxDict.
class TemplateDict(DictBase):

    # Change the HOMEPAGE_URL to the online dictionary you want to add.
    # This will be showed when using `zdict --list-dicts` to help users
    # find out the dictionary they might need.
    HOMEPAGE_URL = "https://tw.dictionary.yahoo.com/"

    # Change the url below to the API url of the new dictionary.
    # Need to keep the `{word}` for `_get_url()` usage.
    API = 'https://tw.dictionary.search.yahoo.com/search?p={word}'

    @property
    def provider(self):
        # Change `template` to the short name of the new dictionary.
        return 'template'

    @property
    def title(self):
        # Change `Template Dictionary` to the title of the new dictionary.
        return 'Template Dictionary'

    def _get_url(self, word) -> str:
        return self.API.format(word=word)

    def show(self, record: Record):
        content = json.loads(record.content)

        # Use `self.color.print()` to render the output.
        # TODO: Add the API doc of the `color.print()`
        print()

        if self.args.verbose:
            try:
                # Get the addtional information if it exists.
                pass
            except Exception:
                return
            else:
                # Define how to print the additional information
                print()

    def query(self, word: str):
        webpage = self._get_raw(word)
        soup = BeautifulSoup(webpage, "html.parser")
        content = {}

        # Parse `data` and fill the information you need into `content`
        #
        # Use
        # ```
        # except AttributeError:
        #    raise NotFoundError(word)
        # ```
        # while the word users try to query is not found on this dictionary.

        if self.args.verbose:
            # For verbose mode
            pass

        record = Record(
            word=word,
            content=json.dumps(content),
            source=self.provider,
        )

        return record



import json
import os
import re

from zdict.constants import BASE_DIR
from zdict.dictionary import DictBase
from zdict.exceptions import NotFoundError, QueryError, APIKeyError
from zdict.models import Record


class OxfordDictionary(DictBase):
    """
    Docs:

    * https://developer.oxforddictionaries.com/documentation/
    """

    KEY_FILE = os.path.join(BASE_DIR, 'oxford.key')

    HOMEPAGE_URL = "https://oxforddictionaries.com/"
    API = 'https://od-api.oxforddictionaries.com/api/v2/entries/en/{word}'

    # https://developer.oxforddictionaries.com/documentation/response-codes
    status_code = {
        200: 'Success!',
        400: 'The request was invalid or cannot be otherwise served.',
        403: 'The request failed due to invalid credentials.',
        404: 'No entry is found.',
        500: 'Something is broken. Please contact the Oxford Dictionaries '
             'API team to investigate.',
        502: 'Oxford Dictionaries API is down or being upgraded.',
        503: 'The Oxford Dictionaries API servers are up, but overloaded '
             'with requests. Please try again later.',
        504: 'The Oxford Dictionaries API servers are up, but the request '
             'couldn’t be serviced due to some failure within our stack. '
             'Please try again later.'
    }

    @property
    def provider(self):
        return 'oxford'

    @property
    def title(self):
        return 'Oxford Dictionary'

    def _get_url(self, word) -> str:
        return self.API.format(word=word.lower())

    def show(self, record: Record):
        content = json.loads(record.content)

        # results
        for headword in content['results']:
            # word
            self.color.print(headword['word'], 'lyellow')

            for lex_ent in headword['lexicalEntries']:
                print()

                # lexical category
                if (
                        'lexicalCategory' in lex_ent
                        and 'text' in lex_ent['lexicalCategory']
                ):
                    self.color.print(
                        lex_ent['lexicalCategory']['text'],
                        'lred',
                        end=''
                    )

                # entry
                idx = 1
                for entry in lex_ent['entries']:
                    # pronunciation
                    if 'pronunciations' in entry:
                        pronunciations = [
                            '/' + pronun['phoneticSpelling'] + '/'
                            for pronun in entry['pronunciations']
                        ]
                        pronunciations_str = '  '.join(pronunciations)
                        self.color.print('  ' + pronunciations_str)
                    else:
                        print()

                    # senses
                    if 'senses' in entry:
                        for sense in entry['senses']:
                            line_prefix = '{idx}.'.format(idx=idx)
                            self._show_sense(sense, line_prefix)
                            idx += 1
                    else:
                        print()

        print()

    def _show_sense(self, sense: dict, prefix='', indent=1):
        print()
        self.color.print(prefix, end=' ', indent=indent)

        # regions
        if 'regions' in sense:
            regions = [region['text'] for region in sense['regions']]
            regions_str = ', '.join(regions)
            regions_str = '(' + regions_str + ')'
            self.color.print(regions_str, 'yellow', end=' ')

        # register
        if 'registers' in sense:
            registers = [register['text'] for register in sense['registers']]
            registers_str = ', '.join(registers)
            self.color.print(registers_str, 'red', end=' ')

        # domain
        if 'domains' in sense:
            domains = [domain['text'] for domain in sense['domains']]
            domains_str = ', '.join(domains)
            domains_str = '(' + domains_str + ') '
            self.color.print(domains_str, 'green', end='')

        # notes
        if 'notes' in sense:
            notes = [note['text'] for note in sense['notes']]
            notes_str = ', '.join(notes)
            notes_str = '[' + notes_str + '] '
            self.color.print(notes_str, 'magenta', end='')

        # definition
        if 'definitions' in sense:
            definition_str = '. '.join(sense['definitions'])
            print(definition_str)

        # cross ref
        if 'crossReferenceMarkers' in sense:
            xref_marker_str = '. '.join(sense['crossReferenceMarkers'])
            print(xref_marker_str)

        # example
        for example in sense.get('examples', []):
            print()
            print(' ' * (indent + 1), end='  ')
            self.color.print(example['text'], 'indigo', indent=indent+2)

        # subsenses
        if self.args.verbose and 'subsenses' in sense:
            for idx, subsense in enumerate(sense['subsenses'], 1):
                line_prefix = '{prefix}{idx}.'.format(prefix=prefix, idx=idx)
                self._show_sense(subsense, line_prefix, indent=indent + 1)

    def _get_app_key(self):
        """
        Get the app id & key for query

        .. note:: app key storage
            The API key should placed in ``KEY_FILE`` in the ``~/.zdict`` with
            the format::

                app_id,app_key

        .. note:: request limit
            request limit: per minute is 60, per month is 3000.
        """
        if not os.path.exists(self.KEY_FILE):
            self.color.print('You can get an API key by the following steps:',
                             'yellow')
            print('1. Register a developer account at '
                  'https://developer.oxforddictionaries.com/')
            print('2. Get the application id & keys in the `credentials` page')
            print('3. Paste the API key at `{key_file}` in the foramt:'.format(
                key_file=self.KEY_FILE
            ))
            print('     app_id, app_key')
            raise APIKeyError('Oxford: API key not found')

        with open(self.KEY_FILE) as fp:
            keys = fp.read()

        keys = re.sub(r'\s', '', keys).split(',')
        if len(keys) != 2:
            print('The API key should be placed in the format:')
            print('     app_id, app_key')
            raise APIKeyError('Oxford: API key file format not correct.')

        return keys

    def query(self, word: str):
        try:
            app_id, app_key = self._get_app_key()
            content = self._get_raw(word, headers={
                'app_id': app_id,
                'app_key': app_key
            })
        except QueryError as exception:
            msg = self.status_code.get(exception.status_code,
                                       'Some bad thing happened')
            self.color.print('Oxford: ' + msg, 'red')
            raise NotFoundError(exception.word)

        record = Record(
            word=word,
            content=content,
            source=self.provider,
        )
        return record


# Format of the Response Value
# ref: <https://developer.oxforddictionaries.com/documentation#/Entries>
"""
{
  "metadata": {},
  "results": [
    {
      "id": "string",
      "language": "string",
      "lexicalEntries": [
        {
          "compounds": [
            {
              "domains": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "id": "string",
              "language": "string",
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "text": "string"
            }
          ],
          "derivativeOf": [
            {
              "domains": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "id": "string",
              "language": "string",
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "text": "string"
            }
          ],
          "derivatives": [
            {
              "domains": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "id": "string",
              "language": "string",
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "text": "string"
            }
          ],
          "entries": [
            {
              "crossReferenceMarkers": [
                "string"
              ],
              "crossReferences": [
                {
                  "id": "string",
                  "text": "string",
                  "type": "string"
                }
              ],
              "etymologies": [
                "string"
              ],
              "grammaticalFeatures": [
                {
                  "id": "string",
                  "text": "string",
                  "type": "string"
                }
              ],
              "homographNumber": "string",
              "inflections": [
                {
                  "domains": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "grammaticalFeatures": [
                    {
                      "id": "string",
                      "text": "string",
                      "type": "string"
                    }
                  ],
                  "inflectedForm": "string",
                  "lexicalCategory": {
                    "id": "string",
                    "text": "string"
                  },
                  "pronunciations": [
                    {
                      "audioFile": "string",
                      "dialects": [
                        "string"
                      ],
                      "phoneticNotation": "string",
                      "phoneticSpelling": "string",
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ]
                    }
                  ],
                  "regions": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "registers": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ]
                }
              ],
              "notes": [
                {
                  "id": "string",
                  "text": "string",
                  "type": "string"
                }
              ],
              "pronunciations": [
                {
                  "audioFile": "string",
                  "dialects": [
                    "string"
                  ],
                  "phoneticNotation": "string",
                  "phoneticSpelling": "string",
                  "regions": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "registers": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ]
                }
              ],
              "senses": [
                {
                  "antonyms": [
                    {
                      "domains": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "id": "string",
                      "language": "string",
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "text": "string"
                    }
                  ],
                  "constructions": [
                    {
                      "domains": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "examples": [
                        [
                          "string"
                        ]
                      ],
                      "notes": [
                        {
                          "id": "string",
                          "text": "string",
                          "type": "string"
                        }
                      ],
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "text": "string"
                    }
                  ],
                  "crossReferenceMarkers": [
                    "string"
                  ],
                  "crossReferences": [
                    {
                      "id": "string",
                      "text": "string",
                      "type": "string"
                    }
                  ],
                  "definitions": [
                    "string"
                  ],
                  "domains": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "etymologies": [
                    "string"
                  ],
                  "examples": [
                    {
                      "definitions": [
                        "string"
                      ],
                      "domains": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "notes": [
                        {
                          "id": "string",
                          "text": "string",
                          "type": "string"
                        }
                      ],
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "senseIds": [
                        "string"
                      ],
                      "text": "string"
                    }
                  ],
                  "id": "string",
                  "inflections": [
                    {
                      "domains": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "grammaticalFeatures": [
                        {
                          "id": "string",
                          "text": "string",
                          "type": "string"
                        }
                      ],
                      "inflectedForm": "string",
                      "lexicalCategory": {
                        "id": "string",
                        "text": "string"
                      },
                      "pronunciations": [
                        {
                          "audioFile": "string",
                          "dialects": [
                            "string"
                          ],
                          "phoneticNotation": "string",
                          "phoneticSpelling": "string",
                          "regions": [
                            {
                              "id": "string",
                              "text": "string"
                            }
                          ],
                          "registers": [
                            {
                              "id": "string",
                              "text": "string"
                            }
                          ]
                        }
                      ],
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ]
                    }
                  ],
                  "notes": [
                    {
                      "id": "string",
                      "text": "string",
                      "type": "string"
                    }
                  ],
                  "pronunciations": [
                    {
                      "audioFile": "string",
                      "dialects": [
                        "string"
                      ],
                      "phoneticNotation": "string",
                      "phoneticSpelling": "string",
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ]
                    }
                  ],
                  "regions": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "registers": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "shortDefinitions": [
                    "string"
                  ],
                  "subsenses": [
                    {}
                  ],
                  "synonyms": [
                    {
                      "domains": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "id": "string",
                      "language": "string",
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "text": "string"
                    }
                  ],
                  "thesaurusLinks": [
                    {
                      "entry_id": "string",
                      "sense_id": "string"
                    }
                  ],
                  "variantForms": [
                    {
                      "domains": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "notes": [
                        {
                          "id": "string",
                          "text": "string",
                          "type": "string"
                        }
                      ],
                      "pronunciations": [
                        {
                          "audioFile": "string",
                          "dialects": [
                            "string"
                          ],
                          "phoneticNotation": "string",
                          "phoneticSpelling": "string",
                          "regions": [
                            {
                              "id": "string",
                              "text": "string"
                            }
                          ],
                          "registers": [
                            {
                              "id": "string",
                              "text": "string"
                            }
                          ]
                        }
                      ],
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "text": "string"
                    }
                  ]
                }
              ],
              "variantForms": [
                {
                  "domains": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "notes": [
                    {
                      "id": "string",
                      "text": "string",
                      "type": "string"
                    }
                  ],
                  "pronunciations": [
                    {
                      "audioFile": "string",
                      "dialects": [
                        "string"
                      ],
                      "phoneticNotation": "string",
                      "phoneticSpelling": "string",
                      "regions": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ],
                      "registers": [
                        {
                          "id": "string",
                          "text": "string"
                        }
                      ]
                    }
                  ],
                  "regions": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "registers": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "text": "string"
                }
              ]
            }
          ],
          "grammaticalFeatures": [
            {
              "id": "string",
              "text": "string",
              "type": "string"
            }
          ],
          "language": "string",
          "lexicalCategory": {
            "id": "string",
            "text": "string"
          },
          "notes": [
            {
              "id": "string",
              "text": "string",
              "type": "string"
            }
          ],
          "phrasalVerbs": [
            {
              "domains": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "id": "string",
              "language": "string",
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "text": "string"
            }
          ],
          "phrases": [
            {
              "domains": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "id": "string",
              "language": "string",
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "text": "string"
            }
          ],
          "pronunciations": [
            {
              "audioFile": "string",
              "dialects": [
                "string"
              ],
              "phoneticNotation": "string",
              "phoneticSpelling": "string",
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ]
            }
          ],
          "text": "string",
          "variantForms": [
            {
              "domains": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "notes": [
                {
                  "id": "string",
                  "text": "string",
                  "type": "string"
                }
              ],
              "pronunciations": [
                {
                  "audioFile": "string",
                  "dialects": [
                    "string"
                  ],
                  "phoneticNotation": "string",
                  "phoneticSpelling": "string",
                  "regions": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ],
                  "registers": [
                    {
                      "id": "string",
                      "text": "string"
                    }
                  ]
                }
              ],
              "regions": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "registers": [
                {
                  "id": "string",
                  "text": "string"
                }
              ],
              "text": "string"
            }
          ]
        }
      ],
      "pronunciations": [
        {
          "audioFile": "string",
          "dialects": [
          "string"
          ],
          "phoneticNotation": "string",
          "phoneticSpelling": "string",
          "regions": [
            {
              "id": "string",
              "text": "string"
            }
          ],
          "registers": [
            {
              "id": "string",
              "text": "string"
            }
          ]
        }
      ],
      "type": "string",
      "word": "string"
    }
  ]
}
"""



# flake8: noqa
from pytest import raises
from unittest.mock import Mock, mock_open, patch

from zdict.dictionaries.oxford import OxfordDictionary
from zdict.exceptions import NotFoundError, QueryError
from zdict.models import Record
from zdict.zdict import get_args


class TestOxfordDictionary:
    def setup_method(self, method):
        self.dict = OxfordDictionary(get_args())

    def teardown_method(self, method):
        del self.dict

    def test_provider(self):
        assert self.dict.provider == 'oxford'

    def test__get_url(self):
        uri = 'https://od-api.oxforddictionaries.com/api/v2/entries/en/mock'
        assert self.dict._get_url('mock') == uri

    @patch('os.path.exists')
    def test__get_app_key(self, exists):
        key_file = mock_open(read_data="""
            test_app_id,
            test_app_key """)

        with patch('builtins.open', key_file):
            app_id, app_key = self.dict._get_app_key()

        assert app_id == 'test_app_id'
        assert app_key == 'test_app_key'

    def test_query_notfound(self):
        self.dict._get_raw = Mock(side_effect=QueryError('mock', 404))
        self.dict._get_app_key = Mock(return_value=('id', 'key'))

        with raises(NotFoundError):
            self.dict.query('mock')

        self.dict._get_raw.assert_called_with('mock', headers={
            'app_id': 'id',
            'app_key': 'key'
        })

    @patch('zdict.dictionaries.oxford.Record')
    def test_query_normal(self, Record):
        self.dict._get_raw = Mock(return_value=SAMPLE_RESPONSE)
        self.dict._get_app_key = Mock(return_value=('id', 'key'))

        self.dict.query('string')

        Record.assert_called_with(word='string',
                                  content=SAMPLE_RESPONSE,
                                  source='oxford')

    def test_show(self):
        r = Record(word='string',
                   content=SAMPLE_RESPONSE,
                   source=self.dict.provider)
        self.dict.show(r)


# the sample response copied from the official website
SAMPLE_RESPONSE = """
{
    "id": "string",
    "metadata": {
        "operation": "retrieve",
        "provider": "Oxford University Press",
        "schema": "RetrieveEntry"
    },
    "results": [
        {
            "id": "string",
            "language": "en-gb",
            "lexicalEntries": [
                {
                    "derivatives": [
                        {
                            "id": "stringless",
                            "text": "stringless"
                        },
                        {
                            "id": "stringlike",
                            "text": "stringlike"
                        }
                    ],
                    "entries": [
                        {
                            "etymologies": [
                                "Old English streng (noun), of Germanic origin; related to German Strang, also to strong. The verb (dating from late Middle English) is first recorded in the senses ‘arrange in a row’ and ‘fit with a string’"
                            ],
                            "pronunciations": [
                                {
                                    "audioFile": "https://audio.oxforddictionaries.com/en/mp3/string_gb_1.mp3",
                                    "dialects": [
                                        "British English"
                                    ],
                                    "phoneticNotation": "IPA",
                                    "phoneticSpelling": "strɪŋ"
                                }
                            ],
                            "senses": [
                                {
                                    "definitions": [
                                        "material consisting of threads of cotton, hemp, or other material twisted together to form a thin length"
                                    ],
                                    "examples": [
                                        {
                                            "text": "unwieldy packs tied up with string"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.006",
                                    "notes": [
                                        {
                                            "text": "mass noun",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "material consisting of threads of cotton, hemp"
                                    ],
                                    "subsenses": [
                                        {
                                            "definitions": [
                                                "a piece of string used to tie round or attach to something"
                                            ],
                                            "examples": [
                                                {
                                                    "text": "the elephant mask had a trunk you could raise by pulling a string"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.008",
                                            "notes": [
                                                {
                                                    "text": "count noun",
                                                    "type": "grammaticalNote"
                                                }
                                            ],
                                            "shortDefinitions": [
                                                "piece of string used to tie round or attach to something"
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "a piece of catgut or similar material interwoven with others to form the head of a sports racket."
                                            ],
                                            "domains": [
                                                {
                                                    "id": "sport",
                                                    "text": "Sport"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.011",
                                            "notes": [
                                                {
                                                    "text": "count noun",
                                                    "type": "grammaticalNote"
                                                }
                                            ],
                                            "shortDefinitions": [
                                                "piece of catgut or similar material interwoven with others"
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "a length of catgut or wire on a musical instrument, producing a note by vibration"
                                            ],
                                            "domains": [
                                                {
                                                    "id": "music",
                                                    "text": "Music"
                                                }
                                            ],
                                            "examples": [
                                                {
                                                    "text": "the D string broke"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.012",
                                            "notes": [
                                                {
                                                    "text": "count noun",
                                                    "type": "grammaticalNote"
                                                }
                                            ],
                                            "shortDefinitions": [
                                                "length of catgut or wire on musical instrument"
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "the stringed instruments in an orchestra"
                                            ],
                                            "domains": [
                                                {
                                                    "id": "instrument",
                                                    "text": "Instrument"
                                                }
                                            ],
                                            "examples": [
                                                {
                                                    "text": "the blend of the wind-group is less perfect than that of the strings"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.013",
                                            "notes": [
                                                {
                                                    "text": "strings",
                                                    "type": "wordFormNote"
                                                }
                                            ],
                                            "shortDefinitions": [
                                                "stringed instruments in orchestra"
                                            ],
                                            "synonyms": [
                                                {
                                                    "language": "en",
                                                    "text": "stringed instruments"
                                                }
                                            ],
                                            "thesaurusLinks": [
                                                {
                                                    "entry_id": "string",
                                                    "sense_id": "t_en_gb0014229.006"
                                                }
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "relating to or consisting of stringed instruments"
                                            ],
                                            "examples": [
                                                {
                                                    "text": "a string quartet"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.015",
                                            "notes": [
                                                {
                                                    "text": "as modifier",
                                                    "type": "grammaticalNote"
                                                }
                                            ],
                                            "shortDefinitions": [
                                                "relating to or consisting of stringed instruments"
                                            ]
                                        }
                                    ],
                                    "synonyms": [
                                        {
                                            "language": "en",
                                            "text": "twine"
                                        },
                                        {
                                            "language": "en",
                                            "text": "cord"
                                        },
                                        {
                                            "language": "en",
                                            "text": "yarn"
                                        },
                                        {
                                            "language": "en",
                                            "text": "thread"
                                        },
                                        {
                                            "language": "en",
                                            "text": "strand"
                                        },
                                        {
                                            "language": "en",
                                            "text": "fibre"
                                        }
                                    ],
                                    "thesaurusLinks": [
                                        {
                                            "entry_id": "string",
                                            "sense_id": "t_en_gb0014229.001"
                                        }
                                    ]
                                },
                                {
                                    "constructions": [
                                        {
                                            "text": "a string of"
                                        }
                                    ],
                                    "definitions": [
                                        "a set of things tied or threaded together on a thin cord"
                                    ],
                                    "examples": [
                                        {
                                            "text": "she wore a string of agates round her throat"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.018",
                                    "shortDefinitions": [
                                        "set of things tied or threaded together on thin cord"
                                    ],
                                    "subsenses": [
                                        {
                                            "constructions": [
                                                {
                                                    "text": "a string of"
                                                }
                                            ],
                                            "definitions": [
                                                "a sequence of similar items or events"
                                            ],
                                            "examples": [
                                                {
                                                    "text": "a string of burglaries"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.019",
                                            "shortDefinitions": [
                                                "sequence of similar items or events"
                                            ],
                                            "synonyms": [
                                                {
                                                    "language": "en",
                                                    "text": "series"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "succession"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "chain"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "sequence"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "concatenation"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "run"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "streak"
                                                }
                                            ],
                                            "thesaurusLinks": [
                                                {
                                                    "entry_id": "string",
                                                    "sense_id": "t_en_gb0014229.003"
                                                }
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "a linear sequence of characters, words, or other data."
                                            ],
                                            "domains": [
                                                {
                                                    "id": "computing",
                                                    "text": "Computing"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.020",
                                            "shortDefinitions": [
                                                "linear sequence of data"
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "a group of racehorses trained at one stable."
                                            ],
                                            "domains": [
                                                {
                                                    "id": "racing",
                                                    "text": "Racing"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.021",
                                            "shortDefinitions": [
                                                "group of racehorses trained at one stable"
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "a reserve team or player holding a specified position in an order of preference"
                                            ],
                                            "domains": [
                                                {
                                                    "id": "sport",
                                                    "text": "Sport"
                                                }
                                            ],
                                            "examples": [
                                                {
                                                    "text": "the village team held Rangers' second string to a 0–0 draw"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.022",
                                            "shortDefinitions": [
                                                "reserve team or player holding specified position in order of preference"
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "a player assigned a specified rank in a team in an individual sport such as squash"
                                            ],
                                            "domains": [
                                                {
                                                    "id": "sport",
                                                    "text": "Sport"
                                                }
                                            ],
                                            "examples": [
                                                {
                                                    "text": "Taylor lost to third string Baines"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.023",
                                            "shortDefinitions": [
                                                "player assigned specified rank in team in individual sport such as squash"
                                            ]
                                        }
                                    ],
                                    "synonyms": [
                                        {
                                            "language": "en",
                                            "text": "strand"
                                        },
                                        {
                                            "language": "en",
                                            "text": "rope"
                                        },
                                        {
                                            "language": "en",
                                            "text": "necklace"
                                        },
                                        {
                                            "language": "en",
                                            "text": "rosary"
                                        },
                                        {
                                            "language": "en",
                                            "text": "chaplet"
                                        }
                                    ],
                                    "thesaurusLinks": [
                                        {
                                            "entry_id": "string",
                                            "sense_id": "t_en_gb0014229.005"
                                        }
                                    ]
                                },
                                {
                                    "definitions": [
                                        "a tough piece of fibre in vegetables, meat, or other food, such as a tough elongated piece connecting the two halves of a bean pod."
                                    ],
                                    "domains": [
                                        {
                                            "id": "food",
                                            "text": "Food"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.025",
                                    "shortDefinitions": [
                                        "tough piece of fibre in food"
                                    ]
                                },
                                {
                                    "definitions": [
                                        "a G-string or thong."
                                    ],
                                    "domains": [
                                        {
                                            "id": "clothing",
                                            "text": "Clothing"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.027",
                                    "shortDefinitions": [
                                        "G-string or thong"
                                    ]
                                },
                                {
                                    "crossReferenceMarkers": [
                                        "short for stringboard"
                                    ],
                                    "crossReferences": [
                                        {
                                            "id": "stringboard",
                                            "text": "stringboard",
                                            "type": "abbreviation of"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.029"
                                },
                                {
                                    "definitions": [
                                        "a hypothetical one-dimensional subatomic particle having the dynamical properties of a flexible loop."
                                    ],
                                    "domains": [
                                        {
                                            "id": "physics",
                                            "text": "Physics"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.031",
                                    "shortDefinitions": [
                                        "hypothetical one-dimensional subatomic particle having dynamical properties of flexible loop"
                                    ],
                                    "subsenses": [
                                        {
                                            "definitions": [
                                                "a hypothetical threadlike concentration of energy within the structure of space–time."
                                            ],
                                            "domains": [
                                                {
                                                    "id": "physics",
                                                    "text": "Physics"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.032",
                                            "shortDefinitions": [
                                                "hypothetical thread-like concentration of energy within structure of space-time"
                                            ],
                                            "variantForms": [
                                                {
                                                    "text": "cosmic string"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ],
                    "language": "en-gb",
                    "lexicalCategory": {
                        "id": "noun",
                        "text": "Noun"
                    },
                    "phrasalVerbs": [
                        {
                            "id": "string_along",
                            "text": "string along"
                        },
                        {
                            "id": "string_out",
                            "text": "string out"
                        },
                        {
                            "id": "string_together",
                            "text": "string together"
                        },
                        {
                            "id": "string_up",
                            "text": "string up"
                        }
                    ],
                    "phrases": [
                        {
                            "id": "how_long_is_a_piece_of_string%3F",
                            "text": "how long is a piece of string?"
                        },
                        {
                            "id": "no_strings_attached",
                            "text": "no strings attached"
                        },
                        {
                            "id": "on_a_string",
                            "text": "on a string"
                        }
                    ],
                    "text": "string"
                },
                {
                    "derivatives": [
                        {
                            "id": "stringless",
                            "text": "stringless"
                        },
                        {
                            "id": "stringlike",
                            "text": "stringlike"
                        }
                    ],
                    "entries": [
                        {
                            "inflections": [
                                {
                                    "grammaticalFeatures": [
                                        {
                                            "id": "pastParticiple",
                                            "text": "Past Participle",
                                            "type": "Non Finiteness"
                                        },
                                        {
                                            "id": "past",
                                            "text": "Past",
                                            "type": "Tense"
                                        }
                                    ],
                                    "inflectedForm": "strung",
                                    "pronunciations": [
                                        {
                                            "audioFile": "https://audio.oxforddictionaries.com/en/mp3/strung_gb_1.mp3",
                                            "dialects": [
                                                "British English"
                                            ],
                                            "phoneticNotation": "IPA",
                                            "phoneticSpelling": "strʌŋ"
                                        }
                                    ]
                                }
                            ],
                            "pronunciations": [
                                {
                                    "audioFile": "https://audio.oxforddictionaries.com/en/mp3/string_gb_1.mp3",
                                    "dialects": [
                                        "British English"
                                    ],
                                    "phoneticNotation": "IPA",
                                    "phoneticSpelling": "strɪŋ"
                                }
                            ],
                            "senses": [
                                {
                                    "definitions": [
                                        "hang (something) so that it stretches in a long line"
                                    ],
                                    "examples": [
                                        {
                                            "text": "lights were strung across the promenade"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.041",
                                    "notes": [
                                        {
                                            "text": "with object and adverbial",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "hang something so that it stretches in long line"
                                    ],
                                    "subsenses": [
                                        {
                                            "definitions": [
                                                "thread (a series of small objects) on a string"
                                            ],
                                            "examples": [
                                                {
                                                    "text": "he collected stones with holes in them and strung them on a strong cord"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.047",
                                            "shortDefinitions": [
                                                "thread series of small objects on string"
                                            ],
                                            "synonyms": [
                                                {
                                                    "language": "en",
                                                    "text": "thread"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "loop"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "link"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "join"
                                                }
                                            ],
                                            "thesaurusLinks": [
                                                {
                                                    "entry_id": "string",
                                                    "sense_id": "t_en_gb0014229.010"
                                                }
                                            ]
                                        },
                                        {
                                            "definitions": [
                                                "be arranged in a long line"
                                            ],
                                            "examples": [
                                                {
                                                    "text": "the houses were strung along the road"
                                                }
                                            ],
                                            "id": "m_en_gbus1003050.048",
                                            "notes": [
                                                {
                                                    "text": "be strung",
                                                    "type": "wordFormNote"
                                                }
                                            ],
                                            "shortDefinitions": [
                                                "be arranged in long line"
                                            ],
                                            "synonyms": [
                                                {
                                                    "language": "en",
                                                    "text": "spread out"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "space out"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "set apart"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "place at intervals"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "distribute"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "extend"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "fan out"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "scatter"
                                                },
                                                {
                                                    "language": "en",
                                                    "text": "straggle"
                                                }
                                            ],
                                            "thesaurusLinks": [
                                                {
                                                    "entry_id": "string_something_out",
                                                    "sense_id": "t_en_gb0014229.014"
                                                }
                                            ]
                                        }
                                    ],
                                    "synonyms": [
                                        {
                                            "language": "en",
                                            "text": "hang"
                                        },
                                        {
                                            "language": "en",
                                            "text": "suspend"
                                        },
                                        {
                                            "language": "en",
                                            "text": "sling"
                                        },
                                        {
                                            "language": "en",
                                            "text": "stretch"
                                        },
                                        {
                                            "language": "en",
                                            "text": "stretch"
                                        },
                                        {
                                            "language": "en",
                                            "text": "sling"
                                        },
                                        {
                                            "language": "en",
                                            "text": "run"
                                        },
                                        {
                                            "language": "en",
                                            "text": "fasten"
                                        },
                                        {
                                            "language": "en",
                                            "text": "tie"
                                        },
                                        {
                                            "language": "en",
                                            "text": "secure"
                                        },
                                        {
                                            "language": "en",
                                            "text": "link"
                                        }
                                    ],
                                    "thesaurusLinks": [
                                        {
                                            "entry_id": "string",
                                            "sense_id": "t_en_gb0014229.008"
                                        },
                                        {
                                            "entry_id": "string",
                                            "sense_id": "t_en_gb0014229.009"
                                        }
                                    ]
                                },
                                {
                                    "definitions": [
                                        "fit a string or strings to (a musical instrument, a racket, or a bow)"
                                    ],
                                    "examples": [
                                        {
                                            "text": "the harp had been newly strung"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.052",
                                    "notes": [
                                        {
                                            "text": "with object",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "fit string or strings to"
                                    ]
                                },
                                {
                                    "definitions": [
                                        "remove the strings from (a bean)."
                                    ],
                                    "id": "m_en_gbus1003050.054",
                                    "notes": [
                                        {
                                            "text": "with object",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "remove strings from"
                                    ]
                                },
                                {
                                    "definitions": [
                                        "hoax or trick (someone)"
                                    ],
                                    "domains": [
                                        {
                                            "id": "billiards",
                                            "text": "Billiards"
                                        }
                                    ],
                                    "examples": [
                                        {
                                            "text": "I'm not stringing you—I'll eat my shirt if it's not true"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.056",
                                    "notes": [
                                        {
                                            "text": "with object",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "regions": [
                                        {
                                            "id": "north_american",
                                            "text": "North_American"
                                        }
                                    ],
                                    "registers": [
                                        {
                                            "id": "informal",
                                            "text": "Informal"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "hoax or trick"
                                    ]
                                },
                                {
                                    "definitions": [
                                        "work as a stringer in journalism"
                                    ],
                                    "domains": [
                                        {
                                            "id": "journalism",
                                            "text": "Journalism"
                                        }
                                    ],
                                    "examples": [
                                        {
                                            "text": "he strings for almost every French radio service"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.058",
                                    "notes": [
                                        {
                                            "text": "no object",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "registers": [
                                        {
                                            "id": "informal",
                                            "text": "Informal"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "work as stringer in journalism"
                                    ]
                                },
                                {
                                    "definitions": [
                                        "determine the order of play by striking the cue ball from baulk to rebound off the top cushion, first stroke going to the player whose ball comes to rest nearer the bottom cushion."
                                    ],
                                    "domains": [
                                        {
                                            "id": "billiards",
                                            "text": "Billiards"
                                        }
                                    ],
                                    "id": "m_en_gbus1003050.060",
                                    "notes": [
                                        {
                                            "text": "no object",
                                            "type": "grammaticalNote"
                                        }
                                    ],
                                    "shortDefinitions": [
                                        "determine order of play by striking cue ball from baulk to rebound off top cushion"
                                    ]
                                }
                            ]
                        }
                    ],
                    "language": "en-gb",
                    "lexicalCategory": {
                        "id": "verb",
                        "text": "Verb"
                    },
                    "phrasalVerbs": [
                        {
                            "id": "string_along",
                            "text": "string along"
                        },
                        {
                            "id": "string_out",
                            "text": "string out"
                        },
                        {
                            "id": "string_together",
                            "text": "string together"
                        },
                        {
                            "id": "string_up",
                            "text": "string up"
                        }
                    ],
                    "phrases": [
                        {
                            "id": "how_long_is_a_piece_of_string%3F",
                            "text": "how long is a piece of string?"
                        },
                        {
                            "id": "no_strings_attached",
                            "text": "no strings attached"
                        },
                        {
                            "id": "on_a_string",
                            "text": "on a string"
                        }
                    ],
                    "text": "string"
                }
            ],
            "type": "headword",
            "word": "string"
        }
    ],
    "word": "string"
}
"""



from pytest import raises
from unittest.mock import Mock, patch
import json

from zdict.dictionaries.wiktionary import WiktionaryDict
from zdict.exceptions import NotFoundError, QueryError
from zdict.models import Record
from zdict.zdict import get_args


class TestWiktionaryDict:
    def setup_method(self, method):
        self.dict = WiktionaryDict(get_args())

    def teardown_method(self, method):
        del self.dict

    def test_provider(self):
        assert self.dict.provider == 'wiktionary'

    def test__get_url(self):
        uri = 'https://en.wiktionary.org/api/rest_v1/page/definition/mock'
        assert self.dict._get_url('mock') == uri

    def test_query_notfound(self):
        self.dict._get_raw = Mock(side_effect=QueryError('mock', 404))

        with raises(NotFoundError):
            self.dict.query('mock')

        self.dict._get_raw.assert_called_with('mock')

    @patch('zdict.dictionaries.wiktionary.Record')
    def test_query_normal(self, Record):
        content = '''
        {"en":[{"partOfSpeech":"part_of_speech",
        "definitions":[{"definition":"definition","examples":["example"]}]}]}
        '''
        self.dict._get_raw = Mock(return_value=content)
        self.dict.query('mock')
        r_content = [{"part_of_speech": "part_of_speech",
                      "definitions": [{"definition": "definition",
                                       "examples": ["example"]}]}]
        Record.assert_called_with(
            word='mock',
            content=json.dumps(r_content),
            source='wiktionary'
        )

    def test_show(self):
        content = '''
        [{"part_of_speech":"part_of_speech",
        "definitions":[{"definition": "definition","examples":["example"]}]}]
        '''

        r = Record(word="string",
                   content=content,
                   source=self.dict.provider)
        self.dict.show(r)



'''
from pytest import raises
from unittest.mock import Mock, patch

from zdict.exceptions import NotFoundError
from zdict.zdict import get_args

# Change TemplateDict to the class of new dictionary and delete this comment
from zdict.dictionaries.template import TemplateDict


# Change TemplateDict to the class of new dictionary and delete this comment
class TestTemplateDict:
    @classmethod
    def setup_class(cls):
        # Change `Template` to the name of the new dict and delete this comment
        cls.dict = TemplateDict(get_args())

        # You may want to change words to some certain test cases.
        cls.words = ['style', 'metadata']

        # Setup normal query data
        cls.dict.args.verbose = False
        cls.records = [cls.dict.query(word) for word in cls.words]

        # Setup verbose query data
        cls.dict.args.verbose = True
        cls.verbose_records = [cls.dict.query(word) for word in cls.words]

        # Change back to default verbose config
        cls.dict.args.verbose = False

    @classmethod
    def teardown_class(cls):
        del cls.dict
        del cls.words
        del cls.records
        del cls.verbose_records

    def test_provider(self):
        # Change `template` to the provider of the new dict and delete this comment
        assert self.dict.provider == 'template'

    def test_title(self):
        # Change `Template Dictionary` to the title of the new dict and delete this comment
        assert self.dict.title == 'Template Dictionary'

    def test__get_url(self):
        # Change url for the new dict and delete this comment
        url = 'https://tw.dictionary.search.yahoo.com/search?p=test'
        assert url == self.dict._get_url('test')

    def test_show(self):
        # god bless this method, hope that it do not raise any exception
        self.dict.args.verbose = False

        for record in self.records:
            self.dict.show(record)

    def test_show_verbose(self):
        # god bless this method, hope that it do not raise any exception
        self.dict.args.verbose = True

        for record in self.verbose_records:
            self.dict.show(record)

    # Change `template` to `{newdict}` for the line below and delete this comment
    @patch('zdict.dictionaries.template.Record')
    def test_query_normal(self, Record):
        self.dict.args.verbose = False

        for i, word in enumerate(self.words):
            self.dict.query(word)
            Record.assert_called_with(
                word=word,
                content=self.records[i].content,
                # Change `template` to the source for the new dictionary and delete this comment
                source='template',
            )

    # Change `template` to `{newdict}` for the line below and delete this comment
    @patch('zdict.dictionaries.template.Record')
    def test_query_verbose(self, Record):
        self.dict.args.verbose = True

        for i, word in enumerate(self.words):
            self.dict.query(word)
            Record.assert_called_with(
                word=word,
                content=self.verbose_records[i].content,
                # Change `template` to `{newdict}` for the line below and delete this comment
                source='template',
            )

    def test_query_not_found(self):
        # Trigger NotFoundError intentionally and see if it works.
        self.dict._get_raw = Mock(return_value='None')
        with raises(NotFoundError):
            self.dict.query(self.words[0])

    # Add other test functions for private functions of the new dictionary class
'''



import json

from bs4 import BeautifulSoup

from zdict.dictionary import DictBase
from zdict.exceptions import NotFoundError, QueryError
from zdict.models import Record


class WiktionaryDict(DictBase):

    HOMEPAGE_URL = "https://en.wiktionary.org/"
    API = 'https://en.wiktionary.org/api/rest_v1/page/definition/{word}'

    @property
    def provider(self):
        return 'wiktionary'

    @property
    def title(self):
        return 'Wiktionary'

    def _get_url(self, word) -> str:
        return self.API.format(word=word)

    def show(self, record: Record):
        content = json.loads(record.content)

        # Render the output.
        self.color.print(record.word, 'lyellow')

        if self.args.verbose:
            for d in content:
                self.color.print(d['part_of_speech'], 'yellow', indent=2)
                for i, defin in enumerate(d['definitions']):
                    self.color.print("{}. {}".format(i+1, defin['definition']),
                                     'org', indent=4)
                    try:
                        defin['examples']
                    except KeyError:
                        pass
                    else:
                        # self.color.print(f"Examples:", 'lindigo', indent=6)
                        for example in defin['examples']:
                            self.color.print(example, 'indigo', indent=6)
        else:
            d = content[0]
            self.color.print(d['part_of_speech'], 'yellow', indent=2)
            self.color.print(d['definitions'][0]['definition'],
                             'org',
                             indent=4)

    def query(self, word: str):
        try:
            content = self._get_raw(word)
        except QueryError as exception:
            raise NotFoundError(exception.word)

        content = json.loads(content)

        try:
            # Get the first definition string from JSON.
            content = content['en']
        except KeyError:
            # API can return JSON that does not contain 'en' language.
            raise NotFoundError(word)

        # Define a list that will be used to create a Record.
        r_content = []

        # For every part of speech append r_content corresponding list.
        for i, d in enumerate(content):
            # Add what part of speech current definitions refers to.
            r_content.append({'part_of_speech': d['partOfSpeech']})

            # Create a list that will store english_definitions
            # of the current part of speech.
            r_content[i]['definitions'] = []

            for j, d2 in enumerate(d['definitions']):
                # Parse definition and append definitions list.
                definition = BeautifulSoup(d2['definition'],
                                           "html.parser").text
                r_content[i]['definitions'].append({'definition': definition})

                # If API provides examples for the current definition
                # create a new list and append them.
                try:
                    d2['examples']
                except KeyError:
                    pass
                else:
                    r_content[i]['definitions'][j]['examples'] = []
                    for ex in d2['examples']:
                        ex = BeautifulSoup(ex, "html.parser").text
                        r_content[i]['definitions'][j]['examples'].append(ex)

        record = Record(
            word=word,
            content=json.dumps(r_content),
            source=self.provider,
        )

        return record



import sys

from zdict.utils import (Color, create_zdict_db_if_not_exists,
                         create_zdict_dir_if_not_exists,
                         import_readline)

from pytest import mark, raises
from unittest.mock import patch


class TestColor:
    def setup_method(self, method):
        self.color = Color()

    def teardown_method(self, method):
        del self.color

    @patch('zdict.utils.sys.stdout.isatty', return_value=True)
    def test_format_in_tty(self, isatty):
        assert '\33[31;1mtest\33[0m' == self.color.format('test', 'lred')
        assert '\33[31mtest\33[0m' == self.color.format('test', 'red')
        assert ('  \33[31mtest\33[0m' ==
                self.color.format('test', 'red', indent=2))
        assert isatty.called

    @patch('zdict.utils.sys.stdout.isatty', return_value=False)
    def test_format_not_tty(self, isatty):
        assert '  test' == self.color.format('test', 'red', indent=2)
        assert isatty.called

    def test_format_default_s(self):
        assert self.color.format('') == ''

    def test_attribute(self):
        self.color.red = '\33[31m'
        self.color.lred = '\33[31;1m'
        with raises(AttributeError):
            self.color.test


@patch('zdict.utils.constants')
@patch('os.mkdir')
@patch('os.path.isdir', return_value=False)
def test_create_zdict_dir_if_not_exists(mkdir, isdir, constants):
    constants.BASE_DIR = '/mock'
    create_zdict_dir_if_not_exists()

    isdir.assert_called_with('/mock')
    mkdir.assert_called_with('/mock')


@patch('builtins.open')
@patch('zdict.utils.constants')
@patch('os.path.exists', return_value=False)
def test_create_zdict_db_if_not_exists(exists, constants, open):
    constants.DB_FILE = '/mock'
    create_zdict_db_if_not_exists()

    exists.assert_called_with('/mock')
    assert open.called


@mark.skipif(sys.platform.startswith('freebsd'),
             reason="gnureadline installation failed on freebsd")
def test_platform_readline():
    '''
    Check the imported readline module on different platforms
    '''
    with patch.object(sys, 'platform', new='linux'):
        readline = import_readline()
        assert readline.__name__ == 'readline'

    with patch.object(sys, 'platform', new='darwin'):
        readline = import_readline()
        expect = 'gnureadline' if sys.version_info <= (3, 5) else 'readline'
        assert readline.__name__ == expect

    with patch.object(sys, 'platform', new='foo'):
        readline = import_readline()
        assert readline.__name__ == 'readline'



import json
import time
from pytest import raises
from unittest.mock import Mock

from zdict.exceptions import NotFoundError
from zdict.zdict import get_args
from zdict.dictionaries.itaigi import iTaigiDict


def _filter_pronounce(d: dict) -> dict:
    new_d = {}
    for k, v in d.items():
        # iTaigiDict sometimes return one item of result
        # with same meaning but different pronounce
        # which will make this unittest randomly fail.
        if k == "pronounce":
            continue

        if isinstance(v, dict):
            new_d[k] = _filter_pronounce(v)
        elif isinstance(v, tuple):
            new_d[k] = tuple(_filter_pronounce(_) for _ in v)
        elif isinstance(v, list):
            new_d[k] = list(_filter_pronounce(_) for _ in v)
        else:
            new_d[k] = v
    return new_d


class TestiTaigiDict:
    @classmethod
    def setup_class(cls):
        cls.dict = iTaigiDict(get_args())

        cls.words = ['芭樂', '測試']

        # Set query_timeout from 5 seconds to 60 seconds,
        # so it won't timeout that often.
        cls.dict.args.query_timeout = 60
        # Setup normal query data
        cls.dict.args.verbose = False
        cls.records = []
        for word in cls.words:
            record = None
            while True:
                try:
                    record = cls.dict.query(word)
                except Exception:
                    # prevent itaigi API 500 error
                    time.sleep(5)
                    continue
                else:
                    cls.records.append(record)
                    break

        # Setup verbose query data
        cls.dict.args.verbose = True
        cls.verbose_records = []
        for word in cls.words:
            record = None
            while True:
                try:
                    record = cls.dict.query(word)
                except Exception:
                    # prevent itaigi API 500 error
                    time.sleep(5)
                    continue
                else:
                    cls.verbose_records.append(record)
                    break

        # Change back to default verbose config
        cls.dict.args.verbose = False

    @classmethod
    def teardown_class(cls):
        del cls.dict
        del cls.words
        del cls.records
        del cls.verbose_records

    def test_provider(self):
        assert self.dict.provider == 'itaigi'

    def test_title(self):
        assert self.dict.title == 'iTaigi - 愛台語'

    def test__get_url(self):
        # Change url for the new dict and delete this comment
        url = 'https://itaigi.tw/平臺項目列表/揣列表?關鍵字=芭樂'
        assert url == self.dict._get_url('芭樂')

    def test_show(self):
        # god bless this method, hope that it do not raise any exception
        self.dict.args.verbose = False

        for record in self.records:
            self.dict.show(record)

    def test_show_verbose(self):
        # god bless this method, hope that it do not raise any exception
        self.dict.args.verbose = True

        for record in self.verbose_records:
            self.dict.show(record)

    def test_query_normal(self):
        self.dict.args.verbose = False

        query_record = None
        for i, word in enumerate(self.words):
            while True:
                try:
                    query_record = self.dict.query(word)
                except Exception:
                    # prevent itaigi API 500 error
                    time.sleep(5)
                    continue
                else:
                    assert (
                        _filter_pronounce(
                            json.loads(query_record.content)
                        ) ==
                        _filter_pronounce(
                            json.loads(self.records[i].content)
                        )
                    )
                    break

    def test_query_verbose(self):
        self.dict.args.verbose = True

        verbose_query_record = None
        for i, word in enumerate(self.words):
            while True:
                try:
                    verbose_query_record = self.dict.query(word)
                except Exception:
                    # prevent itaigi API 500 error
                    time.sleep(5)
                    continue
                else:
                    assert (
                        _filter_pronounce(
                            json.loads(verbose_query_record.content)
                        ) ==
                        _filter_pronounce(
                            json.loads(self.verbose_records[i].content)
                        )
                    )
                    break

    def test_query_not_found(self):
        self.dict._get_raw = Mock(return_value='{"列表": []}')
        with raises(NotFoundError):
            self.dict.query(self.words[0])
        self.dict._get_raw.assert_called_with(self.words[0])



import json

from zdict.dictionary import DictBase
from zdict.exceptions import NotFoundError
from zdict.models import Record


class JishoDict(DictBase):

    HOMEPAGE_URL = "https://jisho.org/"
    API = 'https://jisho.org/api/v1/search/words?keyword={word}'

    @property
    def provider(self):
        # Change `template` to the short name of the new dictionary.
        return 'jisho'

    @property
    def title(self):
        return 'Jisho'

    def _get_url(self, word) -> str:
        return self.API.format(word=word)

    def show(self, record: Record):
        content = json.loads(record.content)

        for i, data in enumerate(content['data']):
            # empty line separation
            if i:
                print()

            # print word
            reading = data['japanese'][0].get('reading', '')
            word = data['japanese'][0].get('word', '')

            if reading:
                self.color.print(reading, 'lyellow')

            if word:
                self.color.print(word, 'yellow')

            for idx, sense in enumerate(data['senses'], 1):

                if sense['parts_of_speech']:
                    self.color.print(
                        ', '.join(sense['parts_of_speech']),
                        'lred',
                    )

                self.color.print(
                    "{}. {}".format(
                        idx,
                        '; '.join(sense['english_definitions'])
                    ),
                    'lgreen',
                    indent=2,
                )

                if sense['see_also']:
                    self.color.print(
                        "See also {}".format(', '.join(sense['see_also'])),
                        'blue',
                        indent=4,
                    )
                if sense['restrictions']:
                    self.color.print(
                        "Only to {}".format(', '.join(sense['restrictions'])),
                        'blue',
                        indent=4
                    )

            # there are other forms for this word.
            if len(data['japanese']) > 1:
                self.color.print('Other forms')
                word_forms = []
                for word_form in data['japanese'][1:]:

                    reading = word_form.get('reading', '')
                    word = word_form.get('word', '')
                    word_forms.append(
                        '{word}[{reading}]'.format(word=word, reading=reading)
                    )

                self.color.print(', '.join(word_forms), 'yellow', indent=2)

            if not self.args.verbose:
                break
        print()

    def query(self, word: str):
        content = self._get_raw(word)
        content_json = json.loads(content)
        if not content_json['data']:
            raise NotFoundError(word)

        record = Record(
            word=word,
            content=content,
            source=self.provider,
        )
        return record



import json
import re
from collections import deque

import bs4

from zdict.dictionary import DictBase
from zdict.exceptions import NotFoundError
from zdict.models import Record


def text(x):
    return x.text


# typing.Callable is new in Python 3.5,
# use type(lambda: None) for function type hints
def foreach(f: type(lambda: None), i: iter) -> None:
    deque(map(f, i), 0)


class YahooDict(DictBase):

    HOMEPAGE_URL = "https://tw.dictionary.yahoo.com/"
    API = 'https://tw.dictionary.yahoo.com/dictionary?p={word}'

    @property
    def provider(self):
        return 'yahoo'

    @property
    def title(self):
        return 'Yahoo Dictionary'

    def _get_url(self, word) -> str:
        return self.API.format(word=word)

    def show(self, record: Record):
        content = json.loads(record.content)
        getattr(self, 'show_v{}'.format(content.get('version', 1)))(content)

    def show_v1(self, content):  # legacy
        # print word
        self.color.print(content['word'], 'yellow')

        # print pronounce
        for k, v in content.get('pronounce', []):
            self.color.print(k, end='')
            self.color.print(v, 'lwhite', end=' ')
        print()

        # print explain
        main_explanations = content.get('explain', [])
        if self.args.verbose:
            main_explanations.extend(content.get('verbose', []))

        for speech in main_explanations:
            self.color.print(speech[0], 'lred')
            for meaning in speech[1:]:
                self.color.print(
                    '{text}'.format(text=meaning[0]),
                    'org',
                    indent=2
                )
                for sentence in meaning[1:]:
                    if sentence:
                        print(' ' * 4, end='')
                        for i, s in enumerate(sentence.split('*')):
                            self.color.print(
                                s,
                                'lindigo' if i % 2 else 'indigo',
                                end=''
                            )
                        print()
        print()

    def show_v2(self, content):
        self.show_v2_summary(content['summary'])
        self.show_v2_explain(content.get('explain'))
        self.show_v2_verbose(content.get('verbose'))
        print()

    def show_v2_summary(self, summary):
        # word
        self.color.print(summary['word'], 'yellow')
        # pronounce
        pronounce = summary.get('pronounce', [])
        for k, v in pronounce:
            self.color.print(k, end='')
            self.color.print(v, 'lwhite', end=' ')
        print() if pronounce else None
        # explain
        indent = True
        for (t, s) in summary.get('explain', []):
            if t == 'explain':
                self.color.print(s, indent=2 * indent)
                indent = True
            elif t == 'pos':
                self.color.print(s, 'lred', end=' ', indent=2 * indent)
                indent = False
        # grammar
        grammar = summary.get('grammar', [])
        print() if grammar else None
        for s in grammar:
            self.color.print(s, indent=2)

    def show_v2_explain(self, explain):
        if not explain:
            return

        print()
        # explain
        for exp in explain:
            type_ = exp['type']
            if type_ == 'PoS':
                self.color.print(exp['text'], 'lred')

            elif type_ == 'item':
                self.color.print(exp['text'], indent=2)
                sentence = exp.get('sentence')

                if not sentence:
                    continue

                indent = True
                for s in sentence:
                    if isinstance(s, str) and s != '\n':
                        self.color.print(s, 'indigo', end='',
                                         indent=indent * 4)
                    elif isinstance(s, list) and s[0] == 'b':
                        self.color.print(s[1], 'lindigo', end='',
                                         indent=indent * 4)
                    elif s == '\n':
                        print()
                        indent = True
                        continue

                    indent = False

    def show_v2_verbose(self, verbose):
        if not self.args.verbose:
            return
        if not verbose:
            return

        print()
        color = {'title': 'lred', 'explain': 'org', 'item': 'indigo'}
        indent = {'title': 0, 'explain': 2, 'item': 4}
        foreach(
            lambda x: self.color.print(x[1], color[x[0]], indent[x[0]]),
            verbose,
        )

    def query(self, word: str):
        webpage = self._get_raw(word)
        data = bs4.BeautifulSoup(webpage, "html.parser")
        content = {}

        # Please bump version if the format changes again.
        # the `show` function will act with respect to version number.

        content['version'] = 2

        # Here are details of each version.
        #
        # The original one, in the old era, there wasn't any concept of
        # version number:
        # content = {
        #     'word': ...,
        #     'pronounce': ...,
        #     'sound': (optional),
        #     'explain': [...],
        #     'verbose': [...],
        # }
        #
        # Verion 2, yahoo dictionary content is provided by Dy.eye
        # at that moment:
        # content = {
        #     'version': 2,
        #     'summary': {
        #         'word': ...,
        #         'pronounce': [('KK', '...'), (...)],  // optional.
        #                                               // e.g. 'google'
        #         'explain': [(optional)],  # 'hospitalized' is summary-only
        #         'grammar': [(optional)],
        #     },
        #     'explain': [...],
        #     'verbose': [(optional)],
        # }

        # Construct summary (required)
        try:
            content['summary'] = self.parse_summary(data, word)
        except AttributeError:
            raise NotFoundError(word)

        # Handle explain (required)
        try:
            content['explain'] = self.parse_explain(data)
        except IndexError:
            raise NotFoundError(word)

        # Extract verbose (optional)
        content['verbose'] = self.parse_verbose(data)

        record = Record(
            word=word,
            content=json.dumps(content),
            source=self.provider,
        )
        return record

    def parse_summary(self, data, word):
        def get_explain(e: bs4.element.Tag):
            def f(ks):
                return (
                    'pos' if 'pos_button' in ks else
                    'explain' if 'dictionaryExplanation' in ks else
                    '?')
            return [
                (f(m.attrs['class']), m.text)
                for n in e.select('ul > li') for m in n.select('div')]

        def get_pronounce(p: bs4.element.Tag):
            return list(
                map(
                    lambda x: re.match(r'(.*)(\[.*\])', x).groups(),
                    p.find('ul').text.strip().split()
                )
            )

        def get_grammar(d: bs4.element.Tag):
            s = ('div#web ol.searchCenterMiddle '
                 'div.dictionaryWordCard > ul > li')
            return list(map(text, d.select(s)))

        node = data.select_one('div#web ol.searchCenterMiddle')
        node = node.select('div.sys_dict_word_card > div.grp-main > div')

        p = None  # optional
        if node is None or len(node) <= 1:  # e.g. "fabor"
            raise NotFoundError(word)
        elif len(node) == 2:  # e.g. "apples"
            w, e = node
        elif len(node) == 3:  # e.g. ?
            w, _, e = node
        elif len(node) == 4:  # e.g. ?
            w, p, _, e = node
        elif len(node) == 5:   # e.g. "metadata"
            w, p, _, _, e = node
        elif len(node) == 6:
            w, p, _, _, _, e = node

        return {
            'word': w.find('span').text.strip(),
            'pronounce': get_pronounce(p) if p else [],  # optional
            'explain': get_explain(e),
            'grammar': get_grammar(data),  # optional
        }

    def parse_explain(self, data):
        def getitem(node):
            # This method will glitch when there are more the one sentence.
            s = node.find_all('span')
            # s should be ['${index}', '${meaning}', '${example_sentence}']
            exp = {
                'type': 'item',
                'text': "{} {}".format(s[0].text, s[1].text),
                'sentence': [],
            }

            for sentence in s[2:]:
                sentence = list(
                    map(
                        lambda x: ('b', x.text) if x.name == 'b' else str(x),
                        sentence.contents
                    )
                )
                if isinstance(sentence[-1], str):
                    hd, _, tl = sentence.pop().rpartition(' ')
                    sentence.extend([hd, '\n', tl])
                sentence.append('\n')
                exp['sentence'].extend(sentence)

            return exp

        ret = []
        explanation_content_node = data.select('div.tab-content-explanation')
        for explanations_node in explanation_content_node:
            for node in explanations_node.children:
                if re.match(r'\d', node.text.strip()):
                    for n in node.find_all('li'):
                        exp = getitem(n)
                        ret.append(exp)
                else:
                    exp = {
                        'type': 'PoS',  # part of speech
                        'text': node.text.strip(),
                    }
                    ret.append(exp)

        return ret

    def parse_verbose(self, data):
        ret = []
        nodes = data.select_one('div.tab-content-synonyms')
        for node in nodes if nodes else []:
            name = node.name
            if name == 'div':
                if node.span is None:
                    continue

                cls = node.span.attrs.get('class')
                if cls is None:
                    continue

                s = node.span.text.strip()
                'fw-xl' in cls and ret.append(('title', s))
                'fw-500' in cls and ret.append(('explain', s))

            elif name == 'ul':
                for li in node.select(':scope > li'):
                    ret.append(('item', li.span.text))
        return ret



from zdict.easter_eggs import (import_pyjokes_module, get_pyjoke)

from unittest.mock import patch


def test_import_pyjokes_module():
    with patch('importlib.util.find_spec', return_value=None):
        assert import_pyjokes_module() is None

    assert import_pyjokes_module().__name__ == 'pyjokes'


def test_get_pyjoke():
    import pyjokes
    with patch('pyjokes.get_jokes', return_value=['test']):
        assert get_pyjoke(pyjokes, 'test').content == 'test'



import sys
import os

from setuptools import find_packages, setup


ROOT_DIR = os.path.dirname(os.path.realpath(__file__))


def parse_requirements(filepath):
    """ load requirements from a pip requirements file. """
    with open(filepath) as f:
        lines = (line.strip() for line in f)
        return [line for line in lines if line and not line.startswith('#')]


def get_zdict_version():
    constants_file_path = os.path.join(ROOT_DIR, 'zdict/constants.py')
    with open(constants_file_path) as constants:
        for line in constants:
            if line.startswith('VERSION'):
                code = compile(line, '<string>', 'single')
                version = code.co_consts[0]

        return version


def get_test_req():
    test_requirements = parse_requirements(
        os.path.join(ROOT_DIR, 'requirements-test.txt')
    )

    if not sys.platform.startswith('freebsd'):
        test_requirements.append('gnureadline==6.3.3')

    return test_requirements


version = get_zdict_version()


install_requirements = parse_requirements(
    os.path.join(ROOT_DIR, 'requirements.txt')
)

if sys.platform == 'darwin' and sys.version_info <= (3, 5):
    install_requirements.append('gnureadline==6.3.3')


setup(
    packages=find_packages(exclude=['scripts']),
    scripts=['scripts/zdict'],
    install_requires=install_requirements,
    tests_require=get_test_req(),

    name='zdict',
    version=version,
    author='Shun-Yi Jheng',
    author_email='M157q.tw@gmail.com',
    maintainer='Shun-Yi Jheng, Iblis Lin, Chang-Yen Chih, Chiu-Hsiang Hsu',
    maintainer_email=('M157q.tw@gmail.com,'
                      'e196819@hotmail.com,'
                      'michael66230@gmail.com,'
                      'wdv4758h@gmail.com'),
    url='https://github.com/zdict/zdict',
    keywords="cli, dictionary, framework",
    description="The last online dictionary framework you need. (?)",
    long_description=open("README.rst", encoding='utf-8').read(),
    long_description_content_type="text/x-rst",
    download_url="https://github.com/zdict/zdict/archive/v{}.zip".format(
        version
    ),
    platforms=['Linux', 'Mac'],
    license="GPL3",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: Chinese (Traditional)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Utilities",
    ],
)



from unittest.mock import patch, Mock

from zdict.api import dump


@patch('zdict.api.Record')
def test_dump(Record):
    # Assume our db contains following words
    Record.select.return_value = [
        Mock(word='apple'),
        Mock(word='apply')]

    # init query
    ret = dump()
    assert ret == ['apple', 'apply']

    ret = dump(pattern='^.*e$')
    assert ret == ['apple']



import os
import sys

from zdict import constants


def create_zdict_dir_if_not_exists():
    if not os.path.isdir(constants.BASE_DIR):
        os.mkdir(constants.BASE_DIR)


def create_zdict_db_if_not_exists():
    if not os.path.exists(constants.DB_FILE):
        open(constants.DB_FILE, 'a').close()


def check_zdict_dir_and_db():
    create_zdict_dir_if_not_exists()
    create_zdict_db_if_not_exists()


class ColorConst(type):
    COLOR_TEMPLATE = "\33[{}m"
    COLOR_LIST = (
        ('ORG', 0),
        ('BLACK', 30),
        ('RED', 31),
        ('GREEN', 32),
        ('YELLOW', 33),
        ('BLUE', 34),
        ('MAGENTA', 35),
        ('INDIGO', 36),
        ('WHITE', 37),
    )

    def __getattr__(cls, color):
        '''
        Magic!
        http://stackoverflow.com/questions/3155436
        '''
        d = dict(cls.COLOR_LIST)
        color = color.upper() if color else 'ORG'
        _color = color if color[0] != 'L' else color[1:]

        if _color not in d.keys():
            raise AttributeError

        return cls.COLOR_TEMPLATE.format(
            '{}{}'.format(
                d.get(_color, 0),
                ';1' if color[0] == 'L' else ''
            )
        )


class Color(metaclass=ColorConst):
    _force_color = False

    @classmethod
    def set_force_color(cls, force_color=True):
        cls._force_color = force_color

    @classmethod
    def format(self, s='', color='org', indent=0):
        '''
        :type s: str
        :param s: message
        :param color: predefined color name, e,g,: red, RED.
            Using 'l' prefix for bright color, e.g.: lred, lwhite.
            It's case-insensitive.

            If stdout isn't a tty, the color option will be ignored.
        '''
        colorize = self._force_color or sys.stdout.isatty()

        return '{indent}{color}{s}{org}'.format(
            indent=' ' * indent,
            color=getattr(self, color, '') if colorize else '',
            s=s,
            org=self.ORG if colorize else '',
        )

    @classmethod
    def print(self, *args, end='\n', **kwargs):
        print(self.format(*args, **kwargs), end=end)


def import_readline():
    if sys.platform == 'darwin' and sys.version_info <= (3, 5):
        import gnureadline as readline
    else:
        import readline
    return readline


readline = import_readline()
